@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2007 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2008 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -24,11 +24,28 @@ from logilab.astng.lookup import builtin_lookup
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
-from pylint.checkers.utils import is_error, is_builtin, is_func_default, \
+from pylint.checkers.utils import is_error, is_builtin, is_func_default, is_func_decorator, \
      is_ancestor_name, assign_parent, are_exclusive, \
      is_defined_before #, is_parent, FOR_NODE_TYPES
 
 
+def overridden_method(klass, name):
+    """get overriden method if any"""
+    try:
+        parent = klass.local_attr_ancestors(name).next()
+    except (StopIteration, KeyError):
+        return None
+    try:
+        meth_node = parent[name]
+    except KeyError:
+        # We have found an ancestor defining <name> but it's not in the local
+        # dictionary. This may happen with astng built from living objects.
+        return None
+    # check its a method
+    if getattr(meth_node, 'argnames', None) is not None:
+        return meth_node
+    return None
+
     
 MSGS = {
     'E0601': ('Using variable %r before assignment',
@@ -115,7 +132,14 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         """visit module : update consumption analysis variable
         checks globals doesn't overrides builtins
         """
-        self._to_consume = [(copy(node.locals), {}, 'module')]
+        mlocals = copy(node.locals)
+        # __dict__ is added to module's locals but not available in module's namespace
+        # (unlike __doc__, __name__, etc...). But take care __dict__ may be assigned
+        # somewhere in the module, so remove astng inserted nodes (having None lineno)
+        mlocals['__dict__'] = [n for n in mlocals['__dict__'] if n.lineno is not None]
+        if not mlocals['__dict__']:
+            del mlocals['__dict__']
+        self._to_consume = [(mlocals, {}, 'module')]
         self._vars = []
         for name, stmts in node.locals.items():
             if name in ('__name__', '__doc__', '__file__', '__path__') \
@@ -192,19 +216,20 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         self._vars.append({})
         
     def leave_function(self, node):
-        """leave function: check function's locals are consumed
-        """
+        """leave function: check function's locals are consumed"""
         not_consumed = self._to_consume.pop()[0]
         self._vars.pop(0)
-        is_method = node.is_method()
-        klass = node.parent.frame()
-        # don't check arguments of abstract methods or within an interface
-        if is_method and (klass.type == 'interface' or node.is_abstract()):
-            return
+        # don't check arguments of function which are only raising an exception
         if is_error(node):
             return
+        # don't check arguments of abstract methods or within an interface
+        is_method = node.is_method()
+        klass = node.parent.frame()
+        if is_method and (klass.type == 'interface' or node.is_abstract()):
+            return
         authorized_rgx = self.config.dummy_variables_rgx
-        for name, stmts in not_consumed.items():
+        overridden = marker = []
+        for name, stmts in not_consumed.iteritems():
             # ignore some special names specified by user configuration
             if authorized_rgx.match(name):
                 continue
@@ -215,9 +240,16 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                 continue
             # care about functions with unknown argument (builtins)
             if node.argnames is not None and name in node.argnames:
-                # don't warn if the first argument of a method is not used
-                if is_method and node.argnames and name == node.argnames[0]:
-                    continue
+                if is_method:
+                    # don't warn for the first argument of a (non static) method
+                    if node.type != 'staticmethod' and \
+                       node.argnames and name == node.argnames[0]:
+                        continue
+                    # don't warn for argument of an overridden method
+                    if overridden is marker:
+                        overridden = overridden_method(klass, node.name)
+                    if overridden is not None and name in overridden.argnames:
+                        continue
                 # don't check callback arguments
                 if node.name.startswith('cb_') or \
                        node.name.endswith('_cb'):
@@ -263,9 +295,14 @@ builtins. Remember that you should avoid to define new builtins when possible.'
 
     def _loopvar_name(self, node, name):
         # filter variables according to node's scope
+        # XXX used to filter parents but don't remember why, and removing this
+        # fixes a W0631 false positive reported by Paul Hachmann on 2008/12 on
+        # python-projects (added to func_use_for_or_listcomp_var test)
+        #astmts = [stmt for stmt in node.lookup(name)[1]
+        #          if hasattr(stmt, 'ass_type')] and
+        #          not stmt.statement().parent_of(node)]
         astmts = [stmt for stmt in node.lookup(name)[1]
-                  if hasattr(stmt, 'ass_type') and
-                  not stmt.statement().parent_of(node)]
+                  if hasattr(stmt, 'ass_type')]
         # filter variables according their respective scope
         if not astmts or astmts[0].statement().parent_of(node):
             _astmts = []
@@ -289,10 +326,10 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         name = node.name
         stmt = node.statement()
         frame = stmt.scope()
-        # if the name node is used as a function default argument's value, then
+        # if the name node is used as a function default argument's value or as a decorator, then
         # start from the parent frame of the function instead of the function
-        # frame
-        if is_func_default(node) or is_ancestor_name(frame, node):
+        # frame - and thus open an inner class scope
+        if is_func_default(node) or is_func_decorator(node) or is_ancestor_name(frame, node):
             start_index = len(self._to_consume) - 2
         else:
             start_index = len(self._to_consume) - 1
